﻿/**
	GameController class
	JDG Communications for Federal Trade Commission
*/

//import com.gaiaframework.debug.*;
//import com.gaiaframework.api.*;

import mx.utils.Delegate;
import gov.ftc.core.Utilities;
import gov.ftc.core.FunctionDelay;

import gov.ftc.squaresgame.GameData;
import gov.ftc.squaresgame.DisplayManager;
import gov.ftc.squaresgame.SoundManager;
import gov.ftc.squaresgame.TicTacToePlayer;
import gov.ftc.squaresgame.Character;

class gov.ftc.squaresgame.GameController extends MovieClip {

// CLASS VARIABLES

	private var __gameData:GameData;
	private var __gd:Object;
	
	private var __displayManager:DisplayManager;
	private var __soundManager:SoundManager;
	
	private var __tttPlayer:TicTacToePlayer;
	
	private var __okToQuit:Boolean;

	public var gameBroadcaster:Object;
	private var __gameListener:Object;
	private var __eventListener:Object;

	private var __destroyList:Array;
	
	
// CONSTRUCTOR

	public function GameController() {
		super();

		_global.squaresgameGC = this;
		
		__gameData = new GameData();
		__gd = _global.squaresgameGD.gameController;
		
		__tttPlayer = new TicTacToePlayer();

		gameBroadcaster = new Object();
		__gameListener = new Object();
		
		AsBroadcaster.initialize(gameBroadcaster);
		gameBroadcaster.addListener(__gameListener);
		
		__gameListener.introDoneEvent = Delegate.create(this, doIntroDone);
		__gameListener.buttonEvent = Delegate.create(this, doButtonEvent);
		__gameListener.screenDisplayDoneEvent = Delegate.create(this, doScreenDisplayDone);
		__gameListener.winAnimationDoneEvent = Delegate.create(this, startEnding);
		__gameListener.questionItemDisplayDoneEvent = Delegate.create(this, doQuestionItemDisplayDone);
		
		__gameListener.characterMouseEvent = Delegate.create(this, doCharacterMouseEvent);
		
		__eventListener = new Object();
				
		//instantiate Display Manager after all controllers
		Utilities.instantiateClassMc(this, "__displayManager", DisplayManager, 100);
		
		Utilities.instantiateClassMc(this, "__soundManager", SoundManager, 110);
		
		__destroyList = [];

		__destroyList.push(
			__gd,
			__displayManager
		);
		
		if (this == _level0) {
//			trace("init at constructor");
			init();
		}
	
	}
	
// CLASS METHODS

	public function init():Void {
		if (_level0.testMode) {
			startGame();
		} else {
			startIntro();
		}
	}
	
	private function startIntro():Void {
//		trace("gc.startIntro");
		__gd.gamePhase = "intro";
		__displayManager.displayScreen("gameIntro");
	}
	
	private function startHostScreen1():Void {
//		trace("gc.startHostScreen1");
		__gd.gamePhase = "hostScreen1";
		__displayManager.displayScreen("hostScreen1");
	}
	
	private function startHostScreen2():Void {
//		trace("gc.startHostScreen2");
		__gd.gamePhase = "hostScreen2";
		__displayManager.displayScreen("hostScreen2");
	}
	
	private function startEnding():Void {
//		trace("gc.startEnding: " + __gd.winner);
		__gd.gamePhase = "ending";
		var screenName:String = (__gd.winner == 1) ? "hostScreen3" : "hostScreen4";
		__displayManager.displayScreen(screenName);
	}
	
	private function doIntroDone():Void {
//		trace("gc.doIntroDone");
		startHostScreen1();
	}
	
	private function doScreenDisplayDone(p_screen:String):Void {
//		trace("gc.doScreenDisplayDone: " + p_screen);
		switch (p_screen) {
			case "gameIntro":
				__displayManager.gameIntro.mc.gotoAndPlay(2);
				break;
		}
	}
	
	private function startGame():Void {
		__gameData.newGame();
		__gd.currentTurn = 2;
		__displayManager.gameBoard.init();
		__displayManager.displayScreen("gameBoard");
		__soundManager.playSound("loop_mix1", true);
		__soundManager.fadeSound("loop_mix1", 0, 100);
		startTurn();
	}
	
	private function startTurn():Void {
		__gd.currentTurn = 3 - __gd.currentTurn;
		__gd.gamePhase = "character";
		__displayManager.gameBoard.poseHost("lookAhead");
		if (__gd.currentTurn == 2) {
			__displayManager.gameBoard.showHostPrompt("standby");
			var opponentCharNum:Number = __tttPlayer.findMove(__gd.gameGrid);
			var opponentChar:Character = __displayManager.gameBoard.mc["character"+opponentCharNum];
			FunctionDelay.callDelayed(opponentChar, "flashBorder", __gd.startTurnFlashBorderDelay * 1000, [true, __gd.currentTurn]);
			FunctionDelay.callDelayed(__soundManager, "playSound", __gd.startTurnPlaySoundDelay * 1000, ["chime3"]);
			FunctionDelay.callDelayed(this, "beginQuestion", __gd.startTurnBeginQuestionDelay * 1000, [opponentChar]);
//			trace(opponentChar);
		} else {
			FunctionDelay.callDelayed(__displayManager.gameBoard, "showHostPrompt", __gd.startTurnShowHostPromptDelay * 1000, ["choose"]);
		}
	}
	
	private function doCharacterMouseEvent(p_event:String, p_mc:Character):Void {
		if (__gd.gamePhase == "character" && __gd.currentTurn == 1) {
			switch (p_event) {
				case "rollOver":
					p_mc.useHandCursor = true;
					p_mc.wiggle("in");
					p_mc.showBorderMode("rolloverX");
					break;
				case "rollOut":
					p_mc.useHandCursor = false;
					p_mc.wiggle("out");
					p_mc.showBorderMode("idle");
					break;
				case "release":
					p_mc.useHandCursor = false;
					p_mc.flashBorder(true, __gd.currentTurn);
					FunctionDelay.callDelayed(this, "beginQuestion", __gd.clickCharacterBeginQuestionDelay * 1000, [p_mc]);
					__soundManager.playSound("chime3");
//					beginQuestion(p_mc);
			}
		}
	}

	private function doButtonEvent(p_event:String, p_mc:MovieClip):Void {
		if (__gd.gamePhase == "question" && p_event == "release") {
			switch (p_mc._name) {
				case "btnContinue":
					__displayManager.question.showStage("answer2", __gd.currentTurn);
					__displayManager.gameBoard.poseHost("lookAhead");
					break;
				case "btnAgree":
					registerAnswer("agree");
					break;
				case "btnDisagree":
					registerAnswer("disagree");
					break;
			}
			return;
		} else if (p_mc._name == "btnContinue" && p_event == "release") {
			switch (__gd.gamePhase) {
				case "answer":
					endQuestion();
					break;
				case "intro":
					startHostScreen1();
					break;
				case "hostScreen1":
					startHostScreen2();
					break;
				case "hostScreen2":
					startGame();
					break;
			}
		} else if (p_mc._name == "btnReplay" && p_event == "release") {
			startGame();
		}
	}

	private function beginQuestion(p_mc:Character):Void {
		__gd.gamePhase = "question";

		__displayManager.gameBoard.dimRest(p_mc);
		p_mc.flashBorder(false);
		__gd.currentCharacter = p_mc;

		//select question
		__gd.currentQuestion = p_mc.id;
		
		//was character used?
		if (__gd.gameGrid[__gd.currentQuestion] == 3) {
			var newQuestionNum = -1;
			for (var i:Number=0; i<__gd.extraQuestionsUsed.length; i++) {
				if (__gd.extraQuestionsUsed[i] == 0) {
					newQuestionNum = 9 + i;
					__gd.extraQuestionsUsed[i] = 1;
				}
			}
			if (newQuestionNum > -1) {
				__gd.currentQuestion = newQuestionNum;
			} else {
				__gd.currentQuestion = 9;
				__gd.extraQuestionsUsed = [1, 0, 0];
			}
		}
		
		__displayManager.question.init(__gd.currentQuestion);
		__displayManager.gameBoard.poseHost("read");
		__displayManager.gameBoard.hideHostPrompt();
	}

	private function doQuestionItemDisplayDone(p_mcName:String):Void {
		switch (p_mcName) {
			case "answer1":
				FunctionDelay.callDelayed(__soundManager, "playBankSound", __gd.playLaughDelay * 1000, ["laughBank"]);
				break;
			case "answer2":
				if (__gd.currentTurn == 2) {
//					trace("opponentAnswer");
					FunctionDelay.callDelayed(this, "doOpponentAnswer", __gd.showOpponentAnswerDelay * 1000, null);
				}
				break;
		}
	}
	
	private function doOpponentAnswer():Void {
		var correct:String = __gd.correctAnswers[__gd.currentQuestion];
		var incorrect:String = (correct == "agree") ? "disagree" : "agree";
		if (Math.random() > 0.5) {
			registerAnswer(correct);
		} else {
			registerAnswer(incorrect);
		}
	}
	
	private function registerAnswer(p_answer:String):Void {
//		trace("gc.registerAnswer: " + p_answer);
		__gd.gamePhase = "answer";
		__displayManager.question.showStage(p_answer, __gd.currentTurn);
		__displayManager.gameBoard.poseHost("lookUp");
		__gd.lastAnswerCorrect = (p_answer == __gd.correctAnswers[__gd.currentQuestion]);
		if (__gd.lastAnswerCorrect) {
			__soundManager.playSound("chime1");
		} else {
			__soundManager.playSound("chime2");
		}
	}
	
	private function endQuestion():Void {
//		trace("gc.endQuestion");
		var promptLbl:String;
		var turnDelay:Number = __gd.endQuestionTurnDelayMark;
		__displayManager.question.end();
		if (__gd.lastAnswerCorrect) {
			__gd.currentCharacter.mark(__gd.currentTurn);
			var winArray:Array = __tttPlayer.getWinMask(__gd.gameGrid, __gd.currentCharacter.id, __gd.currentTurn);
			if (winArray.length) {
				__gd.winner = Number(__gd.currentTurn);
				promptLbl = (__gd.winner == 1) ? "winX" : "winO";
				__displayManager.gameBoard.showHostPrompt(promptLbl);
				__displayManager.gameBoard.showWin(winArray, __gd.currentTurn);
				FunctionDelay.callDelayed(__soundManager, "fadeSound", __gd.endQuestionFadeLoopDelay * 1000, ["loop_mix1", __gd.endQuestionFadeLoopDuration * 1000, 0]);
				FunctionDelay.callDelayed(__soundManager, "stopSound", __gd.endQuestionStopLoopDelay * 1000, ["loop_mix1"]);
				FunctionDelay.callDelayed(__soundManager, "playSound", __gd.endQuestionPlayEndsoundDelay * 1000, ["end_mix01"]);
				return;
			} else {
				__soundManager.playBankSound("applauseBank");
			}
			__gd.gameGrid[__gd.currentCharacter.id] = Number(__gd.currentTurn); 
		} else if (!__tttPlayer.getWinMask(__gd.gameGrid, __gd.currentCharacter.id, 3 - __gd.currentTurn).length) {
			// incorrect answer, but no win - mark goes to non-player
			__gd.gameGrid[__gd.currentCharacter.id] = 3 - __gd.currentTurn;
			__gd.currentCharacter.mark(3 - __gd.currentTurn);
		} else {
			// incorrect answer, mark would win for non-player - no mark
			__gd.gameGrid[__gd.currentCharacter.id] = 3;
			promptLbl = (__gd.currentTurn == 1) ? "noMarkO" : "noMarkX";
			__displayManager.gameBoard.showHostPrompt(promptLbl);
			turnDelay = __gd.endQuestionTurnDelayMiss;
		}
		__displayManager.gameBoard.undimAll();
		FunctionDelay.callDelayed(this, "startTurn", turnDelay * 1000, null);
	}

	public function destroy():Void {
//		trace("gc.destroy");
		FunctionDelay.destroy();
		gameBroadcaster.removeListener(__gameListener);
		var i:Number = __destroyList.length;
		while (i--) {
			__destroyList[i].destroy();
		}
		delete _global.scamblasterGC;
	}
	
}